@@ -1,3 +1,5 @@
+#![feature(fundamental)]
+
 //! *`ed` is a minimalist crate for deterministic binary encodings.*
 //!
 //! ## Overview
@@ -77,6 +79,8 @@ pub use ed_derive::*;
 pub enum Error {
     #[error("Unexpected byte: {0}")]
     UnexpectedByte(u8),
+    #[error("Unencodable variant")]
+    UnencodableVariant,
     #[error(transparent)]
     IOError(#[from] std::io::Error),
 }
@@ -85,6 +89,7 @@ pub enum Error {
 pub type Result<T> = std::result::Result<T, Error>;
 
 /// A trait for values that can be encoded into bytes deterministically.
+#[fundamental]
 pub trait Encode {
     /// Writes the encoded representation of the value to the destination
     /// writer. Can error due to either a write error from `dest`, or an
@@ -116,6 +121,7 @@ pub trait Encode {
 }
 
 /// A trait for values that can be decoded from bytes deterministically.
+#[fundamental]
 pub trait Decode: Sized {
     /// Reads bytes from the reader and returns the decoded value.
     ///
@@ -398,6 +404,11 @@ tuple_impl!(A, B, C; D);
 tuple_impl!(A, B, C, D; E);
 tuple_impl!(A, B, C, D, E; F);
 tuple_impl!(A, B, C, D, E, F; G);
+tuple_impl!(A, B, C, D, E, F, G; H);
+tuple_impl!(A, B, C, D, E, F, G, H; I);
+tuple_impl!(A, B, C, D, E, F, G, H, I; J);
+tuple_impl!(A, B, C, D, E, F, G, H, I, J; K);
+tuple_impl!(A, B, C, D, E, F, G, H, I, J, K; L);
 
 impl<T: Encode + Terminated, const N: usize> Encode for [T; N] {
     #[inline]
@@ -562,6 +573,33 @@ impl<T: Decode> Decode for Box<T> {
     }
 }
 
+impl<T> Encode for std::marker::PhantomData<T> {
+    /// Encoding PhantomData is a no-op.
+    #[inline]
+    #[cfg_attr(test, mutate)]
+    fn encode_into<W: Write>(&self, _: &mut W) -> Result<()> {
+        Ok(())
+    }
+
+    /// Always returns Ok(0).
+    #[inline]
+    #[cfg_attr(test, mutate)]
+    fn encoding_length(&self) -> Result<usize> {
+        Ok(0)
+    }
+}
+
+impl<T> Decode for std::marker::PhantomData<T> {
+    /// Returns a PhantomData without reading any bytes.
+    #[inline]
+    #[cfg_attr(test, mutate)]
+    fn decode<R: Read>(_: R) -> Result<Self> {
+        Ok(Self {})
+    }
+}
+
+impl<T> Terminated for std::marker::PhantomData<T> {}
+
 #[cfg(test)]
 use mutagen::mutate;
 mod tests {
@@ -692,6 +730,16 @@ mod tests {
         assert_eq!(result.unwrap_err().to_string(), "Unexpected byte: 42");
     }
 
+    #[test]
+    fn test_encode_decode_phantom_data() {
+        use std::marker::PhantomData;
+        let pd: PhantomData<u8> = PhantomData;
+        let bytes = pd.encode().unwrap();
+        assert_eq!(bytes.len(), 0);
+        let decoded_value: PhantomData<u8> = Decode::decode(bytes.as_slice()).unwrap();
+        assert_eq!(decoded_value, PhantomData);
+    }
+
     #[test]
     fn test_default_decode() {
         struct Foo {
@@ -828,4 +876,12 @@ mod tests {
         let length = unit.encoding_length().unwrap();
         assert!(length == 0);
     }
+
+    #[test]
+    fn test_phantom_data_encoding_length() {
+        use std::marker::PhantomData;
+        let pd: PhantomData<u8> = PhantomData;
+        let length = pd.encoding_length().unwrap();
+        assert_eq!(length, 0);
+    }
 }
